/*
 * $Id$
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.tiles.autotag.core;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Outputs the files in the test directory.
 */
public class DirectoryOutputLocator implements OutputLocator {

	private File directory;
	private long sourceLastModified;

	public DirectoryOutputLocator(File directory) {
		this.directory = directory;
		this.sourceLastModified = System.currentTimeMillis();
	}

	public DirectoryOutputLocator(File directory, long sourceLastModified) {
		this.directory = directory;
		this.sourceLastModified = sourceLastModified;
	}

	@Override
	public OutputStream getOutputStream(String resourcePath) throws IOException {
		File file = new File(directory, resourcePath);
		file.getParentFile().mkdirs();
		return new FileOutputStream(file);
	}

	@Override
	public boolean isUptodate(String resourcePath) {
		File file = new File(directory, resourcePath);
		return file.exists() && file.lastModified() > sourceLastModified;
	}

}
